# frozen_string_literal: true
# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class EntitiesController < Hermes::Api::Dmo::ApplicationController
        before_action :filter_params, only: :update

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          als_id
          title
          siret
          siren
          is_root
          root_id
          parent_id
          id_rex
          id_cristal
          auto_validation
          auto_request_validation
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          total_spent
          available_points
          applicant_ids
          lessor_id
          group_id
          supervisor_group_id
          manager_group_id
          address
          postal_code
          district
          country
          latitude
          longitude
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          title
          siret
          siren
          id_rex
          id_cristal
          auto_validation
          auto_request_validation
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          applicant_ids
          lessor_id
          address
          postal_code
          district
          country
          latitude
          longitude
        ].freeze
        
        def mutate_params
          MUTATE_PARAMS
        end 

        def can_see_entities
          !(current_user_roles & ['Administrator', 'superadministrateur']).empty?
        end 

        def show 
          return render_json_error(:unauthorized, :cannot_see_entities) unless can_see_entities

          super 
        end 

        def index
          return render_json_error(:unauthorized, :cannot_see_entities) unless can_see_entities

          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          @query_precision ||= {}
  
          paginate(
            index_aggregation(no_acl: true),
            each_serializer: @serializer,
            **@options
          )
        end
          
        def admin_search
          return render json: {status: "unauthorized", code: 401, message: "user is no superadmin"} unless is_super_admin

          aggregation = Hermes::UserRight.all

          if !params['entity_params'].blank?
            entity_query = admin_search_param(
              params: params['entity_params'], 
              possible_params: [:id, :id_cristal, :siren, :siret],
              partial_regex_params: [:title]
            )
            aggregation = aggregation.where(:entity_id.in => Hermes::Entity.where(entity_query).pluck(:id)) 
          end 

          if !params['user_params'].blank?
            user_query = admin_search_param(
              params: params['user_params'], 
              possible_params: [:id, :email, :auth_roles],
              full_regex_params: [:first_name, :last_name]
              )
            aggregation = aggregation.where(:user_id.in => User.where(user_query).pluck(:id)) 
          end 

          paginate_no_root(
            aggregation,
            each_serializer: Hermes::AdminSearchRightSerializer,
            **@options
          )
        end
        
        def is_a_company_collaborator
          current_user_roles.include?('collaborateur offre logement')
          true
        end

        def add_points
          if !params[:points_to_add].nil?
            if is_a_company_collaborator
              points_to_add = params[:points_to_add].to_i
              entity_id = params[:entity_id]
              entity = Hermes::Entity.find(entity_id)
              response_points = entity.add_points(@user, points_to_add)
              render json: response_points
            else 
              render json: {status: "unauthorized", code: 401, message: 'access denied user is no collabroator'}
            end 
          else
            render json: {status: "missing param", code: 404, message: 'no points to add in param'} 
          end 
        end 

        def can_create_tier_account
          Hermes::Entity.readable_by(current_user).find_by(id: @instance.id) && 
          (
            current_user.auth_roles&.include?('entreprise') || 
            current_user.auth_roles&.include?('entreprise administrateur')
          )
        end

        def checks_before_create_account
          @instance = Hermes::Entity.find_by(id: params[:id]) if params[:id]
          return [:not_found, :instance_not_found] unless @instance
          return [:unauthorized, :no_right_on_this_entity] if !can_create_tier_account
          if [:id, :first_name, :last_name, :email, :gender, :phone_number].any? { |p| !params.has_key?(p)  }
            return [:unprocessable_entity, :user_information_missing] 
          end
        end 


        def switch_manual_mode
          if is_super_admin
            entity = Hermes::Entity.find(params[:id])
            if entity.switch_manual_mode
              render json: {status: "success", code: 200, message: "manual mode switch to #{entity.manual_mode}"}
            else
              render json: {status: "not found", code: 404, message: 'entity not found'}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is not a superadmin"}
          end 
        end 

        def generate_user_right_data(users, type)
          users.map { |u| {user_id: u.id.to_s, email: u.email, first_name: u.first_name, last_name: u.last_name, phone_number: u.phone_number, auth_roles: u.auth_roles, right: type}} 
        end

        def user_rights
          return render_json_error(:not_found, :instance_not_found) unless @instance

          if params[:user_id]
            user = User.find(params[:user_id])
            return render_json_error(:not_found, :user_not_found) unless user
            
            right = nil
            
            user.group_ids.each do |id|
              right = 'reader' if id == @instance.group_id
              right = 'user' if id == @instance.supervisor_group_id
              right = 'administrator' if id == @instance.manager_group_id
            end
            
            data = {}
            if right
              data = {
                user_id: user.id.to_s, 
                email: user.email, 
                first_name: user.first_name, 
                last_name: user.last_name, 
                phone_number: user.phone_number, 
                auth_roles: user.auth_roles, 
                right: right
              }
            end
          else
            data = []
            # Fix temporaire avant spération des entity/lessor
            exclude_auth_roles = ['cre', 'rdsl']
  
            readers = User.where(group_ids: @instance.group_id, auth_roles: {"$nin": exclude_auth_roles})          
            users = User.where(group_ids: @instance.supervisor_group_id, auth_roles: {"$nin": exclude_auth_roles})          
            administrator = User.where(group_ids: @instance.manager_group_id, auth_roles: {"$nin": exclude_auth_roles})
            
            data = data + generate_user_right_data(readers, 'reader')
            data = data + generate_user_right_data(users, 'user')
            data = data + generate_user_right_data(administrator, 'administrator')
          end

          render json: data
        end

        def filter_params
          private_fields = %w[auto_validation auto_request_validation]

          if (params[:data][:attributes].keys - private_fields).any? 
            unless @instance && @instance.writable_by?(@user)
              private_fields.each { |field|  params[:data][:attributes].delete field }
            end
          end
        end

        def housing_requests_count
          return render_json_error(:not_found, :instance_not_found) unless @instance

          in_process = Hermes::HousingRequest.where(
            is_active: true,
            entity_id: @instance.id, 
            entity_status: { :$in => ['EN COURS DE SAISIE'] },
            consent_level: 4
          ).count

          delegations = Hermes::HousingRequest.where(
            is_active: true,
            entity_id: @instance.id, 
            entity_status: { :$in => ["A VALIDER", "EN COURS", "EN COURS DE SAISIE"] },
            consent_level: 4
          ).count


          render json: {
            delegations: delegations,
            in_process: in_process
          }
        end
      end
    end
  end
end
