module Hermes
  class HousingRequestInitializer
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::SopraXmlSender
    include Hermes::Concerns::SopraXmlRequestGetter
    RANDOM_LIST_CHAR = '234567890123456789012345678901234567890123456789123abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ'
    
    field :unique_number, type: String
    field :hashed_pin_code, type: String
    field :pin_code, type: String
    field :pin_code_created_at, type: DateTime
    field :email, type: String
    field :done, type: Boolean

    common_indexes *%i[
      unique_number
      hashed_pin_code
      pin_code_created_at
      email
      done
    ]

    
    belongs_to(
      :housing_request,
      class_name: 'Hermes::HousingRequest',
      index: true
    )

    def self.get_all_done
      Hermes::HousingRequestInitializer.where({done: true}).each do |hri|
        ap hri.unique_number
      end
    end
    
    def self.get_all
      Hermes::HousingRequestInitializer.all.each do |hri|
        ap hri.unique_number
      end
    end

    def hidden_email
      return nil unless email

      splited = email.split('@')
      if splited[0].length > 2
        char_count = splited[0].length - 2
        splited[0] = "#{splited[0][0]}#{'*'*char_count}#{splited[0][(char_count + 1)]}"
      else
        domain_splited = splited[1].split('.')
        return email if domain_splited[0].length < 2
        char_count = domain_splited[0].length - 2

        splited[1] = "#{domain_splited[0][0]}#{'*'*char_count}#{domain_splited[0][(char_count + 1)]}.#{domain_splited[1]}"
      end 
      splited.join('@')
    end
    
    def hash_string(str)
      Digest::SHA2.new(512).hexdigest(str)
    end
    
    def format_mail_body(pin_code)
      "Bonjour,

      vous recevez ce mail suite à une tentative de récupération de votre demande sur la base de son numéro unique.
      Afin de confirmer cette récupération pour que votre demande soit dupliquée sur notre site Al'In, veuillez saisir la clé de sécurité dans la zone prévue à cet effet.

      Clé de sécurité : #{pin_code}

      Attention, cette clé de sécurité est valable 24 heures. Passé ce délai il sera nécessaire de recommencer votre démarche.
      Si vous n'êtes pas à l'origine de cette demande, veuillez ne pas prendre en considération ce mail.

      Cordialement

      Toute l'équipe d'Al'in

      ps : ceci est un message automatique, il est inutile de répondre à ce mail.
      "
    end
    
    def generate_pin_code
      random_list_array = RANDOM_LIST_CHAR.split(//)
      pin_code = ''
      
      6.times do |i|
        pin_code += random_list_array[(SecureRandom.random_number * random_list_array.length).floor]
      end

      pin_code
    end 
    
    def send_pin_code_on_mail(user = nil)
      res = get_xml_from_sne(user, unique_number)
      return res if res.is_a? Hash
      return { errors: ['email missing from xml'] } unless email = extract_email_from_xml
     
      pin_code = generate_pin_code
      
      return { errors: ['email not sent'] }  unless Agent.bulk_mail(
        email,
        'securite@masolutionlogement.fr',
        'Récupération demande SNE',
        format_mail_body(pin_code)
      ).deliver_later
      
      update(
        pin_code_created_at: DateTime.current,
        hashed_pin_code: hash_string(pin_code),
        pin_code: pin_code,
        user: user, 
        done: false,
        email: email
      )
      pin_code
    end
    
    def check_pin_code(user_input)
      return false unless hashed_pin_code && pin_code_created_at
      
      hash_string(user_input) == hashed_pin_code && (Time.current - pin_code_created_at) < 24.hours
    end
    
    # a supprimer lors de la merge avec cerfav4 redondant avec la méthode dans le concern sopra_xml_request_getter
    def update_applicant_with_parsed_xml(applicant, parsed_xml)
      if parsed_xml[:workplace_postal_code] != applicant.workplace_postal_code || 
        parsed_xml[:workplace_insee_code] != applicant.workplace_insee_code 
        parsed_xml[:workplace_locality] = nil
      end 
      applicant.update(parsed_xml)
    end 

  
    def hydrate_housing_request_from_sne(user_id) 
      return unless last = last_xml
      parsed_xml = parse_xml(last)
      
      main_applicant = Hermes::Applicant.find_or_create_by(user_id: user_id, main_user_id: user_id)
      housing_request = Hermes::HousingRequest.create(user_id: user_id, main_user_id: user_id, main_applicant: main_applicant, is_active: true)

      Hermes::Nud.create(
        unique_number: unique_number, 
        housing_request_id: housing_request.id, 
        creation_date: parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["dateCreationDemande"]
      )
      
      update_request_and_applicant_from_xml(parsed_xml, housing_request.reload)

      update(
        housing_request_id: housing_request.id.to_s,
        done: true
      )

      housing_request.reload
    end
  end
end
