require 'rails_helper'

RSpec.describe Hermes::Status, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:status)).to be_valid
  end

  describe 'Applicant relationship' do
    let!(:applicant) { create(:applicant) }
    let!(:applicant2) { create(:applicant, status: 'Created') }
    let!(:status) do
      create(
        :status,
        object_id: applicant.id.to_s,
        status_label: 'status',
        title: 'Test', 
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: applicant.id.to_s,
        status_label: 'status',
        title: 'Changed',
      )
    end

    it 'set up properly' do
      expect(status.reference_model_name).to eq('Hermes::Applicant')
      expect(status.status_label).to eq('status')
      expect(applicant2.reload.status).to eq('Created')
    end

    it 'Changed status on applicant & changed status' do
      expect(applicant.reload.status).to eq('Changed')
      expect(applicant.reload.old_status).to eq('Test')
      expect(applicant.reload.status_id).to eq(status2.id)
      expect(applicant.reload.status_updated_at.to_date).to eq(status2.reload.created_at.to_date)
      expect(status2.reload.closing).to eq(status.id)
      expect(status.reload.closed_by).to eq(status2.id)
    end
    
    it 'Direct change on applicant status' do
      applicant.status = "Direct change"
      applicant.save
      new_status = Hermes::Status.where("applicant_id": applicant.id, status_label: 'status').order(created_at: :desc).limit(1).first
      expect(applicant.reload.status).to eq('Direct change')
      expect(applicant.reload.old_status).to eq('Changed')
      expect(Hermes::Status.all().length).to eq(4)
      expect(new_status.title).to eq('Direct change')
      expect(new_status.created_at.to_date).to eq(applicant.status_updated_at.to_date)
      expect(new_status.reload.closing).to eq(status2.id)
    end
  end

  describe 'Housing offer relationship' do
    let!(:housing_offer) { create(:housing_offer) }
    let!(:status) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'lessor_status',
        title: 'Test'
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'lessor_status',
        title: 'Changed'
      )
    end

    let!(:status3) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'offer_status',
        title: 'Statut offer init'
      )
    end

    let!(:status4) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'entity_status',
        performer_id: '1',
        title: 'Statut offer init'
      )
    end

    let!(:status5) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'entity_status',
        performer_id: '2',
        title: 'Statut offer init'
      )
    end

    let!(:status6) do
      create(
        :status,
        object_id: housing_offer.id.to_s,
        status_label: 'entity_status',
        performer_id: '2',
        title: 'Changed'
      )
    end

    it 'set up properly' do
      expect(status.reference_model_name).to eq('Hermes::HousingOffer')
      expect(status.status_label).to eq('lessor_status')
    end

    it 'Changed lessor_status and offer_status on housing offer & changed status' do
      expect(housing_offer.reload.lessor_status).to eq('Changed')
      expect(housing_offer.reload.offer_status).to eq('Statut offer init')
      expect(housing_offer.reload.old_lessor_status).to eq('Test')
      expect(housing_offer.reload.lessor_status_id).to eq(status2.id)
      expect(housing_offer.reload.lessor_status_updated_at.to_date).to eq(status2.reload.created_at.to_date)
      expect(status2.reload.closing).to eq(status.id)
      expect(status.reload.closed_by).to eq(status2.id)
      expect(status3.reload.closing).to eq(nil)
      expect(housing_offer.entity_status.length).to eq(2)
      expect(housing_offer.entity_status[1]['closing']).to_not be_nil
      expect(Hermes::Status.all().count).to eq(6)
    end

    ['lessor_status', 'offer_status'].each do |status_label|
      describe "Direct change #{status_label}" do

        before :each do
          housing_offer.send("#{status_label}=", "Direct change")
          housing_offer.save
        end
        
        it 'create statuts' do
          new_status = Hermes::Status.where("housing_offer_id": housing_offer.id, status_label: status_label).order(created_at: :desc).limit(1).first
          old = ['Statut offer init', 'Test', 'Changed']
          expect(housing_offer.reload[status_label]).to eq('Direct change')
          expect(old.include?(housing_offer.reload.send("old_#{status_label}"))).to be true
          expect(new_status.title).to eq('Direct change')
          expect(new_status.created_at.to_date).to eq(housing_offer.send("#{status_label}_updated_at").to_date)
          expect(new_status.reload.closing).to_not be_nil
          expect(Hermes::Status.all().count).to eq(7)
        end
      end
    end
  end

  describe 'Housing request relationship' do
    let!(:applicant) { create(:applicant) }
    let!(:housing_request) { create(:housing_request, main_applicant: applicant) }
    let!(:status) do
      create(
        :status,
        object_id: housing_request.id.to_s,
        status_label: 'entity_status',
        title: 'Test', 
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: housing_request.id.to_s,
        status_label: 'entity_status',
        title: 'Changed',
      )
    end

    let!(:status3) do
      create(
        :status,
        object_id: housing_request.id.to_s,
        status_label: 'request_status',
        title: 'Statut request init',
      )
    end

    it 'set up properly' do
      expect(status.reference_model_name).to eq('Hermes::HousingRequest')
      expect(status.status_label).to eq('entity_status')
    end

    it 'Changed entity_status and request_status on housing request' do
      expect(housing_request.reload.entity_status).to eq('Changed')
      expect(housing_request.reload.request_status).to eq('Statut request init')
      expect(housing_request.reload.old_entity_status).to eq('Test')
      expect(housing_request.reload.entity_status_id).to eq(status2.id)
      expect(housing_request.reload.entity_status_updated_at).to eq(status2.reload.created_at)
      expect(status2.reload.closing).to eq(status.id)
      expect(status.reload.closed_by).to eq(status2.id)
      #expect(status3.reload.closing).to eq(nil)
    end
  end

  describe 'Entity relationship' do
    let!(:entity) { create(:entity) }
    let!(:status) do
      create(
        :status,
        object_id: entity.id.to_s,
        status_label: 'status',
        title: 'Test'
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: entity.id.to_s,
        status_label: 'status',
        title: 'Changed'
      )
    end

    it 'set up properly' do
      expect(status.reference_model_name).to eq('Hermes::Entity')
      expect(status.status_label).to eq('status')
    end

    it 'Changed status on entity' do
      expect(entity.reload.status).to eq('Changed')
      expect(entity.reload.old_status).to eq('Test')
      expect(entity.reload.status_id).to eq(status2.id)
      expect(entity.reload.status_updated_at).to eq(status2.reload.created_at)
      expect(status2.reload.closing).to eq(status.id)
      expect(status.reload.closed_by).to eq(status2.id)
    end
  end

  describe 'Lessor relationship' do
    let!(:lessor) { create(:lessor) }
    let!(:status) do
      create(
        :status,
        object_id: lessor.id.to_s,
        status_label: 'status',
        title: 'Test'
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: lessor.id.to_s,
        status_label: 'status',
        title: 'Changed'
      )
    end

    it 'set up properly' do
      expect(status.reference_model_name).to eq('Hermes::Lessor')
      expect(status.status_label).to eq('status')
    end

    it 'Changed status on lessor' do
      expect(lessor.reload.status).to eq('Changed')
      expect(lessor.reload.old_status).to eq('Test')
      expect(lessor.reload.status_id).to eq(status2.id)
      expect(lessor.reload.status_updated_at).to eq(status2.reload.created_at)
      expect(status2.reload.closing).to eq(status.id)
      expect(status.reload.closed_by).to eq(status2.id)
    end
  end
end
